import { fromString, toString } from "uint8arrays";
import { MIN_MESSAGE_ID, encodeId } from "./ids.js";
export class MerkleIndex {
    constructor(db) {
        Object.defineProperty(this, "db", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: db
        });
    }
    async *entries(options = {}) {
        let lowerBound = { gte: MIN_MESSAGE_ID };
        while (true) {
            const results = await this.db.query("$messages", {
                orderBy: { id: "asc" },
                select: { id: true, hash: true },
                where: { id: lowerBound },
                limit: options.pageSize ?? MerkleIndex.defaultPageSize,
            });
            if (results.length === 0) {
                break;
            }
            for (const { id, hash } of results) {
                yield [encodeId(id), { hash: fromString(hash, "hex") }];
            }
            lowerBound = { gt: results[results.length - 1].id };
        }
    }
}
Object.defineProperty(MerkleIndex, "defaultPageSize", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: 4096
});
Object.defineProperty(MerkleIndex, "encodeNodeId", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: (level, key) => {
        const l = level.toString(16).padStart(2, "0");
        if (key === null) {
            return l;
        }
        else {
            const k = toString(key, "hex");
            return `${l}:${k}`;
        }
    }
});
Object.defineProperty(MerkleIndex, "encodeNode", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: (node) => {
        const id = MerkleIndex.encodeNodeId(node.level, node.key);
        return { id, hash: node.hash };
    }
});
Object.defineProperty(MerkleIndex, "decodeNode", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: ({ id, hash }) => {
        const [l, k] = id.split(":");
        const level = parseInt(l, 16);
        const key = k ? fromString(k, "hex") : null;
        return { level, key, hash };
    }
});
